@@ -1,5 +1,4 @@
 //go:build unix
-// +build unix
 
 // Package nfs implements a server to serve a VFS remote over NFSv3 protocol
 //
@@ -92,7 +91,7 @@ Where ` + "`$PORT`" + ` is the same port number we used in the serve nfs command
 
 This feature is only available on Unix platforms.
 
-` + vfs.Help,
+` + vfs.Help(),
 	Annotations: map[string]string{
 		"versionIntroduced": "v1.65",
 		"groups":            "Filter",
